@@ -7,12 +7,15 @@ use base qw(Exporter);
 @JSON::EXPORT = qw(from_json to_json jsonToObj objToJson encode_json decode_json);
 
 BEGIN {
-    $JSON::VERSION = '2.21_001';
+    $JSON::VERSION = '2.49_01';
     $JSON::DEBUG   = 0 unless (defined $JSON::DEBUG);
+    $JSON::DEBUG   = $ENV{ PERL_JSON_DEBUG } if exists $ENV{ PERL_JSON_DEBUG };
 }
 
 my $Module_XS  = 'JSON::XS';
 my $Module_PP  = 'JSON::PP';
+my $Module_bp  = 'JSON::backportPP'; # included in JSON distribution
+my $PP_Version = '2.27008';
 my $XS_Version = '2.27';
 
 
@@ -42,6 +45,7 @@ my $_INSTALL_DONT_DIE  = 1; # When _load_xs fails to load XS, don't die.
 my $_INSTALL_ONLY      = 2; # Don't call _set_methods()
 my $_ALLOW_UNSUPPORTED = 0;
 my $_UNIV_CONV_BLESSED = 0;
+my $_USSING_bpPP       = 0;
 
 
 # Check the environment variable to decide worker module. 
@@ -60,6 +64,10 @@ unless ($JSON::Backend) {
     elsif ($backend eq '2' or $backend eq 'JSON::XS') {
         _load_xs();
     }
+    elsif ($backend eq 'JSON::backportPP') {
+        $_USSING_bpPP = 1;
+        _load_pp();
+    }
     else {
         Carp::croak "The value of environmental variable 'PERL_JSON_BACKEND' is invalid.";
     }
@@ -128,6 +136,9 @@ sub objToJson {
 # INTERFACES
 
 sub to_json ($@) {
+    if ( ref($_[0]) eq 'JSON' or $_[0] eq 'JSON' ) {
+        Carp::croak "to_json should not be called as a method.";
+    }
     my $json = new JSON;
 
     if (@_ == 2 and ref $_[1] eq 'HASH') {
@@ -142,6 +153,9 @@ sub to_json ($@) {
 
 
 sub from_json ($@) {
+    if ( ref($_[0]) eq 'JSON' or $_[0] eq 'JSON' ) {
+        Carp::croak "from_json should not be called as a method.";
+    }
     my $json = new JSON;
 
     if (@_ == 2 and ref $_[1] eq 'HASH') {
@@ -178,7 +192,7 @@ sub is_xs {
 
 
 sub is_pp {
-    return $_[0]->module eq $Module_PP;
+    return not $_[0]->xs;
 }
 
 
@@ -258,26 +272,43 @@ sub _load_xs {
 
 sub _load_pp {
     my $opt = shift;
+    my $backend = $_USSING_bpPP ? $Module_bp : $Module_PP;
 
-    $JSON::DEBUG and Carp::carp "Load $Module_PP.";
+    $JSON::DEBUG and Carp::carp "Load $backend.";
 
     # if called after install module, overload is disable.... why?
     JSON::Boolean::_overrride_overload($Module_XS);
-    JSON::Boolean::_overrride_overload($Module_PP);
+    JSON::Boolean::_overrride_overload($backend);
+
+    if ( $_USSING_bpPP ) {
+        eval qq| require $backend |;
+    }
+    else {
+        eval qq| use $backend $PP_Version () |;
+    }
 
-    eval qq| require $Module_PP |;
     if ($@) {
-        Carp::croak $@;
+        if ( $backend eq $Module_PP ) {
+            $JSON::DEBUG and Carp::carp "Can't load $Module_PP ($@), so try to load $Module_bp";
+            $_USSING_bpPP++;
+            $backend = $Module_bp;
+            JSON::Boolean::_overrride_overload($backend);
+            local $^W; # if PP installed but invalid version, backportPP redifines methods.
+            eval qq| require $Module_bp |;
+        }
+        Carp::croak $@ if $@;
     }
 
     unless (defined $opt and $opt & $_INSTALL_ONLY) {
-        _set_module( $JSON::Backend = $Module_PP );
+        _set_module( $JSON::Backend = $Module_PP ); # even if backportPP, set $Backend with 'JSON::PP'
         JSON::Backend::PP->init;
     }
 };
 
 
 sub _set_module {
+    return if defined $JSON::true;
+
     my $module = shift;
 
     local $^W;
@@ -346,7 +377,7 @@ package JSON::Backend::PP;
 
 sub init {
     local $^W;
-    no strict qw(refs);
+    no strict qw(refs); # this routine may be called after JSON::Backend::XS init was called.
     *{"JSON::decode_json"} = \&{"JSON::PP::decode_json"};
     *{"JSON::encode_json"} = \&{"JSON::PP::encode_json"};
     *{"JSON::PP::is_xs"}  = sub { 0 };
@@ -405,6 +436,18 @@ sub support_by_pp {
     local $^W;
     no strict qw(refs);
 
+    my $JSON_XS_encode_orignal     = \&JSON::XS::encode;
+    my $JSON_XS_decode_orignal     = \&JSON::XS::decode;
+    my $JSON_XS_incr_parse_orignal = \&JSON::XS::incr_parse;
+
+    *JSON::XS::decode     = \&JSON::Backend::XS::Supportable::_decode;
+    *JSON::XS::encode     = \&JSON::Backend::XS::Supportable::_encode;
+    *JSON::XS::incr_parse = \&JSON::Backend::XS::Supportable::_incr_parse;
+
+    *{JSON::XS::_original_decode}     = $JSON_XS_decode_orignal;
+    *{JSON::XS::_original_encode}     = $JSON_XS_encode_orignal;
+    *{JSON::XS::_original_incr_parse} = $JSON_XS_incr_parse_orignal;
+
     push @JSON::Backend::XS::Supportable::ISA, 'JSON';
 
     my $pkg = 'JSON::Backend::XS::Supportable';
@@ -442,21 +485,6 @@ sub support_by_pp {
 
 package JSON::Backend::XS::Supportable;
 
-{
-    my $JSON_XS_encode_orignal = \&JSON::XS::encode;
-    my $JSON_XS_decode_orignal = \&JSON::XS::decode;
-    my $JSON_XS_incr_parse_orignal = \&JSON::XS::incr_parse;
-
-    local $^W;
-    *JSON::XS::decode = \&JSON::Backend::XS::Supportable::_decode;
-    *JSON::XS::encode = \&JSON::Backend::XS::Supportable::_encode;
-    *JSON::XS::incr_parse = \&JSON::Backend::XS::Supportable::_incr_parse;
-
-    *{JSON::XS::_original_decode} = $JSON_XS_decode_orignal;
-    *{JSON::XS::_original_encode} = $JSON_XS_encode_orignal;
-    *{JSON::XS::_original_incr_parse} = $JSON_XS_incr_parse_orignal;
-}
-
 $Carp::Internal{'JSON::Backend::XS::Supportable'} = 1;
 
 sub _make_unsupported_method {
@@ -484,7 +512,8 @@ sub _make_unsupported_method {
 
 
 sub _set_for_pp {
-    require JSON::PP;
+    JSON::_load_pp( $_INSTALL_ONLY );
+
     my $type  = shift;
     my $pp    = new JSON::PP;
     my $prop = $_[0]->property;
@@ -608,11 +637,22 @@ JSON - JSON (JavaScript Object Notation) encoder/decoder
  
 =head1 VERSION
 
-    2.21
+    2.50
 
 This version is compatible with JSON::XS B<2.27> and later.
 
 
+=head1 PLAN
+
+JSON::PP was inculded in JSON distribution (CPAN module).
+It comes to be a perl core module in Perl 5.14.
+And JSON::PP well be split away it.
+
+This developer-versoin is for the preparation.
+
+    * JSON distribution will inculde yet another JSON::PP modules.
+      They are JSNO::backportPP. So JSON.pm should work as it did at all!
+
 =head1 DESCRIPTION
 
  ************************** CAUTION ********************************
